@@ -5,6 +5,8 @@
 //!
 //! It is in development and the API is not stable.  Please see the [source repository README.md page](https://github.com/chrissimpkins/commandlines-rust) for updates on the level of library support for the POSIX/GNU command line argument syntax.
 
+#![cfg_attr(feature = "cargo-clippy", allow(new_without_default_derive))]
+
 pub mod parsers;
 
 use std::collections::HashMap;
@@ -99,11 +101,8 @@ impl Command {
     //
     // # Arguments
     //
-    // - arguments: (`Vec<String>`) - a Vector of ordered String items intended to mock command line arguments
-    //
-    // Used for `Command` struct testing with mock command line argument data and is not part of the public API.
-    // Use `Command::new()` where `std::env::args().collect()` is used to automatically populate the command line
-    // argument data presented to an executable
+    // - arguments: (`Vec<String>`) - a Vector of ordered String items
+    #[cfg_attr(feature = "cargo-clippy", allow(needless_pass_by_value))]
     fn new_with_vec(arguments: Vec<String>) -> Self {
         let arguments_definition = arguments.to_owned();
         let executable_definition = &arguments[0];
@@ -167,11 +166,36 @@ impl Command {
         !self.definitions.is_empty()
     }
 
+    /// Returns a boolean for the question "Does the command include any multi-option short syntax style option arguments?"
+    ///
+    /// # Remarks
+    /// POSIX defines a short option style that uses a single dash delimiter with more than one option indicated by the individual characters defined in the argument string (e.g., `-hij` means that the command has the options `-h -i -j`).  This method provides support for determining whether a mops style option is present in the command string.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// let c = commandlines::Command::new();
+    ///
+    /// if c.has_mops() {
+    ///     // at least one multi-option short syntax style argument was parsed in the command
+    /// }
+    /// ```
+    pub fn has_mops(&self) -> bool {
+        for arg in &self.options {
+            if parsers::is_mops_option(&arg[..]) {
+                return true;
+            }
+        }
+        false
+    }
+
     /// Returns a boolean for the question "Does the command include any options?"
     ///
     /// # Remarks
     /// An option is defined as a command line argument that starts with one or two hyphen characters. This definition includes standard long (e.g., `--longoption`) and short (e.g., `-s`) command line options.
     ///
+    /// If you use POSIX multi-option short syntax style arguments (e.g., "-lmn" is used to indicate "-l -m -n") in your application, see the `Command::contains_mops()` method.  This method does not test against mops style command line arguments.
+    ///
     /// # Examples
     ///
     /// ```
@@ -218,7 +242,30 @@ impl Command {
         }
     }
 
-    /// Returns a boolean for the question "Does the command include the option string `needle`?" at any index
+    /// Returns a boolean for the question "Does the command include the option `needle` when the POSIX multi-option short syntax option style is used?"
+    ///
+    /// # Remarks
+    /// The mops style defined by POSIX includes a single dash character followed by one or more alphanumeric characters in the Unicode Basic Latin set.  Each character represents a unique option switch.  For example, "-lmn" is used to indicate "-l -m -n".  This method tests against any short option formatted argument included in the command irrespective of the number of alphanumeric characters that are included.  The method does not test against long options (i.e., those that begin with two dashes).
+    ///
+    /// If you do not use the mops option syntax in your application, use the `Command::contains_option()` method instead.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// let c = commandlines::Command::new();
+    ///
+    /// if c.contains_mops("-j") {
+    ///     // the `j` switch was identified in a short format option on the command line
+    /// }
+    /// ```
+    pub fn contains_mops(&self, needle: &str) -> bool {
+        match parsers::parse_mops(&self.options) {
+            Some(haystack) => haystack.contains(&String::from(needle)),
+            None => false,
+        }
+    }
+
+    /// Returns a boolean for the question "Does the command include the option string `needle` at any index?"
     ///
     /// # Examples
     ///
@@ -233,6 +280,38 @@ impl Command {
         self.options.contains(&String::from(needle))
     }
 
+    /// Returns a boolean for the question "Does the command include the option `needle` when the POSIX multi-option short syntax option style is used?"
+    ///
+    /// # Remarks
+    /// The mops style defined by POSIX includes a single dash character followed by one or more alphanumeric characters in the Unicode Basic Latin set.  Each character represents a unique option switch.  For example, "-lmn" is used to indicate "-l -m -n".  This method tests against any short option formatted argument included in the command irrespective of the number of alphanumeric characters that are included.  The method does not test against long options (i.e., those that begin with two dashes).
+    ///
+    /// If you do not use the mops option syntax in your application, use the `Command::contains_all_options()` method instead.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// let c = commandlines::Command::new();
+    ///
+    /// if c.contains_all_mops(vec!["-j", "-k", "-l"]) {
+    ///     // the `j`, `k` and `l` switches were identified in short format options on the command line
+    /// }
+    /// ```
+    pub fn contains_all_mops(&self, needle_vec: Vec<&str>) -> bool {
+        let some_haystack = parsers::parse_mops(&self.options);
+        if some_haystack.is_some() {
+            let haystack = some_haystack.unwrap();
+            for needle in needle_vec {
+                if !haystack.contains(&String::from(needle)) {
+                    return false;
+                }
+            }
+        } else {
+            return false; // there were no mops parsed in the command so there is no match
+        }
+
+        true
+    }
+
     /// Returns a boolean for the question "Does the command include all of the option strings in `needle_vec` Vector?"
     ///
     /// # Examples
@@ -253,6 +332,37 @@ impl Command {
         true
     }
 
+    /// Returns a boolean for the question "Does the command include the option `needle` when the POSIX multi-option short syntax option style is used?"
+    ///
+    /// # Remarks
+    /// The mops style defined by POSIX includes a single dash character followed by one or more alphanumeric characters in the Unicode Basic Latin set.  Each character represents a unique option switch.  For example, "-lmn" is used to indicate "-l -m -n".  This method tests against any short option formatted argument included in the command irrespective of the number of alphanumeric characters that are included.  The method does not test against long options (i.e., those that begin with two dashes).
+    ///
+    /// If you do not use the mops option syntax in your application, use the `Command::contains_any_options()` method instead.
+    ///
+    /// # Examples
+    /// ```
+    /// let c = commandlines::Command::new();
+    ///
+    /// if c.contains_any_mops(vec!["-j", "-k", "-l"]) {
+    ///     // the `j` or `k` or `l` switch was identified in short format options on the command line
+    /// }
+    /// ```
+    pub fn contains_any_mops(&self, needle_vec: Vec<&str>) -> bool {
+        let some_haystack = parsers::parse_mops(&self.options);
+        if some_haystack.is_some() {
+            let haystack = some_haystack.unwrap();
+            for needle in needle_vec {
+                if haystack.contains(&String::from(needle)) {
+                    return true;
+                }
+            }
+        } else {
+            return false; // there were no mops parsed in the command so there is no match
+        }
+
+        false
+    }
+
     /// Returns a boolean for the question "Does the command include any of the option strings in the `needle_vec` Vector?"
     ///
     /// # Examples
@@ -376,7 +486,7 @@ mod tests {
     #[test]
     fn command_instantiation_environment_args_test() {
         let c = Command::new();
-        assert!(c.argv.len() > 0); // should always be a Vector with length 1 or more
+        assert!(!c.argv.is_empty()); // should always be a Vector with length 1 or more
     }
 
     #[test]
@@ -400,7 +510,7 @@ mod tests {
     #[test]
     fn command_instantiation_executable_field() {
         let c = Command::new_with_vec(vec!["test".to_string(), "--help".to_string()]);
-        assert!(c.executable == "test".to_string());
+        assert!(c.executable == "test");
     }
 
     #[test]
@@ -531,16 +641,40 @@ mod tests {
         assert_eq!(c.has_definitions(), false);
     }
 
+    #[test]
+    fn command_has_mops_true() {
+        let c = Command::new_with_vec(vec![
+            String::from("command"),
+            String::from("-lmn"),
+            String::from("lastpos"),
+        ]);
+        assert_eq!(c.has_mops(), true);
+    }
+
+    #[test]
+    fn command_has_mops_false() {
+        let c = Command::new_with_vec(vec![
+            String::from("command"),
+            String::from("--long"),
+            String::from("-o"),
+            String::from("path"),
+            String::from("lastpos"),
+            String::from("--"),
+            String::from("-"),
+        ]);
+        assert_eq!(c.has_mops(), false);
+    }
+
     #[test]
     fn command_method_has_options_true() {
         let c = Command::new_with_vec(vec!["test".to_string(), "--help".to_string()]);
-        assert!(c.has_options() == true);
+        assert_eq!(c.has_options(), true);
     }
 
     #[test]
     fn command_method_has_options_false() {
         let c = Command::new_with_vec(vec!["test".to_string(), "subcmd".to_string()]);
-        assert!(c.has_options() == false);
+        assert_eq!(c.has_options(), false);
     }
 
     #[test]
@@ -571,6 +705,44 @@ mod tests {
         assert_eq!(c.contains_definition("--help"), false);
     }
 
+    #[test]
+    fn command_method_contains_mops_true() {
+        let c = Command::new_with_vec(vec![
+            "test".to_string(),
+            "subcmd".to_string(),
+            "-hij".to_string(),
+            "-l".to_string(),
+            "--option=definition".to_string(),
+            "--another=deftwo".to_string(),
+            "lastpos".to_string(),
+        ]);
+        assert_eq!(c.contains_mops("-h"), true);
+        assert_eq!(c.contains_mops("-i"), true);
+        assert_eq!(c.contains_mops("-j"), true);
+        assert_eq!(c.contains_mops("-l"), true); // should pick up every short option, including those that are not mops formatted
+        assert_eq!(c.contains_mops("-z"), false);
+        assert_eq!(c.contains_mops("-o"), false);
+        assert_eq!(c.contains_mops("-a"), false);
+    }
+
+    #[test]
+    fn command_method_contains_mops_false() {
+        let c = Command::new_with_vec(vec![
+            "test".to_string(),
+            "subcmd".to_string(),
+            "--option=definition".to_string(),
+            "--another=deftwo".to_string(),
+            "lastpos".to_string(),
+            "--".to_string(),
+            "-hij".to_string(),
+        ]);
+        assert_eq!(c.contains_mops("-o"), false);
+        assert_eq!(c.contains_mops("-a"), false);
+        assert_eq!(c.contains_mops("-h"), false); // should ignore all options after a double dash idiom
+        assert_eq!(c.contains_mops("-i"), false); // should ignore all options after a double dash idiom
+        assert_eq!(c.contains_mops("-j"), false); // should ignore all options after a double dash idiom
+    }
+
     #[test]
     fn command_method_contains_option() {
         let c = Command::new_with_vec(vec![
@@ -583,6 +755,46 @@ mod tests {
         assert_eq!(c.contains_option("help"), false); // must include the option indicator in string
     }
 
+    #[test]
+    fn command_method_contains_all_mops_true() {
+        let c = Command::new_with_vec(vec![
+            "test".to_string(),
+            "subcmd".to_string(),
+            "-hij".to_string(),
+            "-l".to_string(),
+            "--option=definition".to_string(),
+            "--another=deftwo".to_string(),
+            "lastpos".to_string(),
+        ]);
+        assert_eq!(c.contains_all_mops(vec!["-h", "-i", "-j", "-l"]), true);
+    }
+
+    #[test]
+    fn command_method_contains_all_mops_false() {
+        let c = Command::new_with_vec(vec![
+            "test".to_string(),
+            "subcmd".to_string(),
+            "-hij".to_string(),
+            "-l".to_string(),
+            "--option=definition".to_string(),
+            "--another=deftwo".to_string(),
+            "lastpos".to_string(),
+        ]);
+        assert_eq!(c.contains_all_mops(vec!["-z", "-h"]), false);
+    }
+
+    #[test]
+    fn command_method_contains_all_mops_missing_mops() {
+        let c = Command::new_with_vec(vec![
+            "test".to_string(),
+            "subcmd".to_string(),
+            "--option=definition".to_string(),
+            "--another=deftwo".to_string(),
+            "lastpos".to_string(),
+        ]);
+        assert_eq!(c.contains_all_mops(vec!["-i", "-j"]), false);
+    }
+
     #[test]
     fn command_method_contains_all_options() {
         let c = Command::new_with_vec(vec![
@@ -597,6 +809,34 @@ mod tests {
         assert_eq!(c.contains_all_options(vec!["subcmd"]), false); // not an option, should not be included in test
     }
 
+    #[test]
+    fn command_method_contains_any_mops() {
+        let c = Command::new_with_vec(vec![
+            "test".to_string(),
+            "subcmd".to_string(),
+            "-hij".to_string(),
+            "-l".to_string(),
+            "--option=definition".to_string(),
+            "--another=deftwo".to_string(),
+            "lastpos".to_string(),
+        ]);
+        assert_eq!(c.contains_any_mops(vec!["-z", "-t", "-h"]), true);
+        assert_eq!(c.contains_any_mops(vec!["-z", "-t", "-l"]), true);
+        assert_eq!(c.contains_any_mops(vec!["-z", "-t", "-a"]), false);
+    }
+
+    #[test]
+    fn command_method_contains_any_mops_missing_mops() {
+        let c = Command::new_with_vec(vec![
+            "test".to_string(),
+            "subcmd".to_string(),
+            "--option=definition".to_string(),
+            "--another=deftwo".to_string(),
+            "lastpos".to_string(),
+        ]);
+        assert_eq!(c.contains_any_mops(vec!["-z", "-t", "-h"]), false);
+    }
+
     #[test]
     fn command_method_contains_any_option() {
         let c1 = Command::new_with_vec(vec![
